import java.util.*;
/**
 * Clase que representa una cola de espera de un servicio
 */
public class ColaEspera
{

    /**
     *  Clase interna para almacenar todos los datos 
     *  de un cliente
     */
    private class DatosCliente {
        Cliente c;
        long entrada, salida; // milisegundos
        
        /**
         * Constructor de DatosCliente; pone la hora de entrada
         */
        DatosCliente (Cliente c) {
            this.c=c;
            entrada=Reloj.ahora();
        }
        
        void atiende() {
            salida=Reloj.ahora();
        }
    }

    // colas del servicio
    private Queue<DatosCliente> colaEspera;
    private Queue<DatosCliente> colaAtendidos;
    

    /**
     * Constructor de ColaEspera
     */
    public ColaEspera()
    {
        colaEspera=new LinkedList<DatosCliente>();
        colaAtendidos=new LinkedList<DatosCliente>();
    }

    /**
     * Nuevo cliente; se mete en la cola de espera
     */
    public void nuevoCliente(Cliente c)
    {
        DatosCliente datos=new DatosCliente(c);
        colaEspera.add(datos);
    }
    
    /**
     * Atender cliente: se saca de la cola de espera
     * y se mete en la de atendidos; retorna el cliente
     * atendido
     */
    public Cliente atenderCliente() throws NoSuchElementException
    {
        DatosCliente datos=colaEspera.remove();
        datos.atiende(); 
        colaAtendidos.add(datos);
        return datos.c;
    }
    
    /**
     * Tiempo de espera medio para clientes en espera (en segundos)
     */
    public double tiempoEsperaNoAtendidos()
    {
        long tiempo=0;
        int num=0;
        long ahora=Reloj.ahora();
        for (DatosCliente datos: colaEspera) {
            tiempo=tiempo + ahora-datos.entrada;
            num++;
        }
        if (num==0) {
            return 0.0;
        } else {
            return (((double) tiempo)/num)/1000.0;
        }
    }
    
    /**
     * Tiempo de espera medio para clientes atendidos (en segundos)
     */
    public double tiempoEsperaAtendidos()
    {
        long tiempo=0;
        int num=0;
        for (DatosCliente datos: colaAtendidos) {
            tiempo=tiempo+datos.salida-datos.entrada;
            num++;
        }
        if (num==0) {
            return 0.0;
        } else {
            return (((double) tiempo)/num)/1000.0;
        }
    }
    
    /**
     * Tiempo de espera maximo para clientes atendidos (en segundos)
     */
    public double tiempoMaxEsperaAtendidos() {
        long tiempoMax=0;
        long tiempo;
        for (DatosCliente datos: colaAtendidos) {
            tiempo=datos.salida-datos.entrada;
            if (tiempo>tiempoMax) {
                tiempoMax=tiempo;
            }
        }
        return ((double)tiempoMax)/1000.0;
    }
    
    /**
     * Mostrar el estado de la cola de espera
     */
    public void muestraEstado() {
        System.out.println();
        System.out.println("-------Estado de la cola de espera----");
        System.out.println("No atendidos "+colaEspera.size());
        if (colaEspera.size()>0) {
            System.out.println("Tiempo medio espera no atendidos(seg): "+
                               tiempoEsperaNoAtendidos());
        }
        System.out.println("Atendidos "+colaAtendidos.size());
        if (colaAtendidos.size()>0) {
            System.out.println("Tiempo medio espera atendidos (seg): "+
                               tiempoEsperaAtendidos());
            System.out.println("Tiempo maximo espera atendidos (seg): "+
                               tiempoMaxEsperaAtendidos());
        }
        System.out.println();
    }
    
    /**
     * Num clientes en espera
     */
    public int numClientesEnEspera() {
        return colaEspera.size();
    }
}
